using System;
using System.Collections;
using System.IO;
using System.Reflection;
using System.Text;
using System.Xml;
using System.Xml.Schema;
using System.Xml.Serialization;
using System.Windows.Forms;
using Kodama.BootLoader.Schema;
using Kodama.Script.Engine;
using Kodama.Script.Factory;
using Kodama.Script.Engine.CodeDom.CSharpScript;
using Kodama.Script.Engine.CodeDom.JScript;
using Kodama.Script.Engine.CodeDom.VBScript;

namespace Kodama.BootLoader
{
    /// <summary>
    /// X^hA[AvP[Ṽu[g[_
    /// </summary>
    /// <remarks>
    /// <p>u[g[_͎st@CłBȉ̂悤ȓ܂B</p>
    /// <p>1.[Uu[g[_N</p>
    /// <p>2.u[g[_R}hCɓnꂽXNvgZbgAbv</p>
    /// <p>3.XNvg̃Gg[|CgĂяo</p>
    /// <p>4.XNvgDependencyInjectionReiɃR|[lg̓o^A
    ///   vpeB̃oChs</p>
    /// <p>5.Reiɓo^ꂽR|[lg̃Gg[|Cg(Np̃C^[tFCX)
    ///   XNvgĂяo</p>
    /// <br/>
    /// <p>u[g[_̃R}hC͈ȉ̂悤ɂȂĂ܂B</p>
    /// <li>-s XNvgt@C̎w(ws)BXNvgVBScriptAJScriptAC#p\</li>
    /// <li>-c XNvgŎQƂAZuꂽXMLt@C̎w(ws)</li>
    /// <p>-sw͕K{łA-c̓IvVłBΈȉ̂悤ȋNp[^ƂȂ܂B</p>
    /// <code>
    /// StandAloneBootLoader -s boot.vb -c ref.xml
    /// </code>
    /// <p>XNvgŎQƂAZuꂽXMLt@C̕\LłA
    /// ȉ̂悤ɂȂĂ܂B</p>
    /// <code>
    /// &lt;?xml version="1.0" encoding="shift_jis"?&gt;
    /// &lt;referenceAssembly&gt;
    ///     &lt;assembly name="foo.dll"/&gt;
    ///     &lt;assembly name="bar.dll"/&gt;
    /// &lt;/referenceAssembly&gt;
    /// </code>
    /// <p>ڍׂ"Kodama\BootLoader\Schema\ReferenceAssembly.xsd"QƁB</p>
    /// </remarks>
    public class StandAloneBootLoader
    {
        /// <summary>
        /// X^hA[AvP[ṼGg[|Cgł
        /// </summary>
        /// <param name="args">R}hC</param>
        [STAThread]
        private static void Main(string[] args)
        {
            // R}hCp[X
            string scriptPath = null;
            string configPath = null;
            for (int i = 0; i < args.Length; ++i)
            {
                if ((string.Compare(args[i], "-s", true) == 0) && (i + 1 < args.Length))
                {
                    scriptPath = args[i + 1];
                }
                else if ((string.Compare(args[i], "-c", true) == 0) && (i + 1 < args.Length))
                {
                    configPath = args[i + 1];
                }
            }

            // XNvg̎w肪Ȃꍇ̓G[
            if (scriptPath == null)
            {
                MessageBox.Show("Please specify a script to set a command line argument.");
                return;
            }

            // XMLݒt@Cp[X
            referenceAssembly reference = null;
            if (configPath != null)
            {
                Stream stream = Assembly.GetExecutingAssembly().GetManifestResourceStream
                    ("Kodama.BootLoader.Schema.AssemblyConfig.xsd");
                XmlSchemaCollection schema = new XmlSchemaCollection();
                schema.Add(null, new XmlTextReader(stream));

                XmlValidatingReader reader = new XmlValidatingReader(new XmlTextReader(configPath));
                reader.ValidationType = ValidationType.Schema;
                reader.Schemas.Add(schema);

                XmlSerializer serializer = new XmlSerializer(typeof(referenceAssembly));

                try
                {
                    reference = (referenceAssembly)serializer.Deserialize(reader);
                }
                finally
                {
                    reader.Close();
                }
            }

            // XNvgGW쐬AXNvgĂяo
            IScriptEngineFactory[] factories = new IScriptEngineFactory[]
                {new CSharpScriptEngineFactory(), new JScriptEngineFactory(), new VBScriptEngineFactory()};
            foreach (IScriptEngineFactory factory in factories)
            {
                // T|[gĂtH[}bg`FbN
                if (!factory.CanCompile(scriptPath))
                {
                    continue;
                }

                // XNvgGW̍쐬
                IScriptEngine engine = factory.CreateFromFile(scriptPath);

                // XMLt@CɏĂAZu̎QƂݒ肷
                if (reference != null)
                {
                    foreach (referenceAssemblyAssembly assembly in reference.Items)
                    {
                        engine.AddReference(assembly.name);
                    }
                }

                // RpC
                try
                {
                    engine.Compile();
                }
                catch (CompileErrorException e)
                {
                    StringBuilder errors = new StringBuilder(10000);
                    foreach (CompileErrorInfo cei in e.GetCompileErrorInfos())
                    {
                        errors.Append("--------------------------------" + "\n");
                        errors.Append("SourceName  : " + cei.SourceName  + "\n");
                        errors.Append("Description : " + cei.Description + "\n");
                        errors.Append("Line        : " + cei.ErrorLine   + "\n");
                        errors.Append("Code        : " + cei.ErrorText   + "\n");
                        errors.Append("--------------------------------" + "\n");
                    }
                    engine.Close();
                    MessageBox.Show("The error occurred in compile of the script.\n" + errors.ToString());
                    return;
                }

                // XNvgs
                try
                {
                    engine.Run();
                }
                catch (ScriptEntryPointNotFoundException)
                {
                    engine.Close();
                    MessageBox.Show("An entry point is not found in the script.");
                }

                // GW
                engine.Close();

                return;
            }

            MessageBox.Show("It is the format of the script which is not supported.");
        }
    }
}
