using System;
using System.Collections;
using System.Reflection;
using Kodama.Function.Functor;
using Kodama.Function.Functor.Bind;
using Kodama.Function.Functor.Member;
using Kodama.DependencyInjection.Container;
using Kodama.DependencyInjection.Marker;

namespace Kodama.DependencyInjection.Component
{
    /// <summary>
    /// DependencyInjectionReiɓo^ĂR|[lg
    /// </summary>
    /// <remarks>
    /// ̃R|[lgœo^ĂR|[lǵA
    /// IComponentContainer#GetComponentɏɐVCX^XԂB
    /// </remarks>
    /// <seealso cref="Kodama.DependencyInjection.Container.ComponentContainerImpl"/>
    public class PrototypeComponentEntry : IComponentEntry
    {
        /// <summary>
        /// o^ĂR|[lǧ^
        /// </summary>
        private Type componentType;

        /// <summary>
        /// o^ĂR|[lg̖O
        /// </summary>
        private string componentName;

        /// <summary>
        /// ˑ𒍓邽߂̃RXgN^oCht@N^
        /// </summary>
        private BindFunctor injectionConstructorFunctor;

        /// <summary>
        /// ˑ𒍓邽߂̈oChς݂̊֐IuWFNg
        /// </summary>
        private ArrayList injectionFactors = new ArrayList();

        /// <summary>
        /// R|[lg邽߂̈oChς݂̊֐IuWFNg
        /// </summary>
        private ArrayList initializationFactors = new ArrayList();

        /// <summary>
        /// CX^X̐ǂ̃tOBzQƂôɂ
        /// </summary>
        private bool instantiating = false;

        /// <summary>
        /// o^ĂR|[lǧ^
        /// </summary>
        public Type ComponentType
        {
            get { return componentType; }
        }

        /// <summary>
        /// o^ĂR|[lg̖O
        /// </summary>
        public string ComponentName
        {
            get { return componentName; }
        }

        /// <summary>
        /// ˑ𒍓邽߂̃RXgN^̈oCht@N^
        /// </summary>
        public BindFunctor InjectionConstructor
        {
            set { injectionConstructorFunctor = value; }
        }

        /// <summary>
        /// RXgN^
        /// </summary>
        /// <param name="container">R|[lgo^Rei</param>
        /// <param name="type">o^R|[lǧ^</param>
        public PrototypeComponentEntry(IComponentContainer container, Type type)
        {
            componentType = type;
            componentName = type.FullName;

            injectionConstructorFunctor = null;
            foreach (ConstructorInfo ci in componentType.GetConstructors())
            {
                if (Attribute.IsDefined(ci, typeof(InjectionPointAttribute)))
                {
                    ArrayList arguments = new ArrayList();
                    foreach (ParameterInfo param in ci.GetParameters())
                    {
                        if (Attribute.IsDefined(param, typeof(ExplicitComponentAttribute)))
                        {
                            ExplicitComponentAttribute eca =
                                (ExplicitComponentAttribute)Attribute.GetCustomAttribute
                                (param, typeof(ExplicitComponentAttribute));
                            arguments.Add(eca.CreateProvider(container));
                        }
                        else
                        {
                            arguments.Add(new TypedArgumentComponentProvider(container, param.ParameterType));
                        }    
                    }
                    injectionConstructorFunctor = new BindFunctor(new ConstructorFunctor(ci), arguments.ToArray());
                }
            }

            foreach (MethodInfo mi in componentType.GetMethods())
            {
                if (Attribute.IsDefined(mi, typeof(InjectionPointAttribute)))
                {
                    ArrayList arguments = new ArrayList();
                    arguments.Add(new NotBoundArgument(0));
                    foreach (ParameterInfo param in mi.GetParameters())
                    {
                        if (Attribute.IsDefined(param, typeof(ExplicitComponentAttribute)))
                        {
                            ExplicitComponentAttribute eca =
                                (ExplicitComponentAttribute)Attribute.GetCustomAttribute
                                (param, typeof(ExplicitComponentAttribute));
                            arguments.Add(eca.CreateProvider(container));
                        }
                        else
                        {
                            arguments.Add(new TypedArgumentComponentProvider(container, param.ParameterType));
                        }    
                    }
                    injectionFactors.Add(new BindFunctor(new MemberFunctor(mi), arguments.ToArray()));
                }
            }

            foreach (MethodInfo mi in componentType.GetMethods())
            {
                if (Attribute.IsDefined(mi, typeof(InitializationPointAttribute)))
                {
                    initializationFactors.Add(new MemberFunctor(mi));
                }
            }
        }

        /// <summary>
        /// RXgN^
        /// </summary>
        /// <param name="container">R|[lgo^Rei</param>
        /// <param name="type">o^R|[lǧ^</param>
        /// <param name="name">o^R|[lg̖O</param>
        public PrototypeComponentEntry(IComponentContainer container, Type type, string name)
        {
            componentType = type;
            componentName = name;

            injectionConstructorFunctor = null;
            foreach (ConstructorInfo ci in componentType.GetConstructors())
            {
                if (Attribute.IsDefined(ci, typeof(InjectionPointAttribute)))
                {
                    ArrayList arguments = new ArrayList();
                    foreach (ParameterInfo param in ci.GetParameters())
                    {
                        if (Attribute.IsDefined(param, typeof(ExplicitComponentAttribute)))
                        {
                            ExplicitComponentAttribute eca =
                                (ExplicitComponentAttribute)Attribute.GetCustomAttribute
                                (param, typeof(ExplicitComponentAttribute));
                            arguments.Add(eca.CreateProvider(container));
                        }
                        else
                        {
                            arguments.Add(new TypedArgumentComponentProvider(container, param.ParameterType));
                        }    
                    }
                    injectionConstructorFunctor = new BindFunctor(new ConstructorFunctor(ci), arguments.ToArray());
                }
            }

            foreach (MethodInfo mi in componentType.GetMethods())
            {
                if (Attribute.IsDefined(mi, typeof(InjectionPointAttribute)))
                {
                    ParameterInfo[] paramerters = mi.GetParameters();
                    ArrayList       arguments   = new ArrayList();
                    arguments.Add(new NotBoundArgument(0));
                    foreach (ParameterInfo param in mi.GetParameters())
                    {
                        if (Attribute.IsDefined(param, typeof(ExplicitComponentAttribute)))
                        {
                            ExplicitComponentAttribute explicitComponentAttribute =
                                (ExplicitComponentAttribute)Attribute.GetCustomAttribute
                                (param, typeof(ExplicitComponentAttribute));
                            arguments.Add(explicitComponentAttribute.CreateProvider(container));
                        }
                        else
                        {
                            arguments.Add(new TypedArgumentComponentProvider(container, param.ParameterType));
                        }    
                    }
                    injectionFactors.Add(new BindFunctor(new MemberFunctor(mi), arguments.ToArray()));
                }
            }

            foreach (MethodInfo mi in componentType.GetMethods())
            {
                if (Attribute.IsDefined(mi, typeof(InitializationPointAttribute)))
                {
                    initializationFactors.Add(new MemberFunctor(mi));
                }
            }
        }

        /// <summary>
        /// ˑ𒍓邽߂̈oChς݂̊֐IuWFNgǉ
        /// </summary>
        /// <param name="functor">ǉ֐IuWFNg</param>
        public void AddInjectionFanctor(BindFunctor functor)
        {
            injectionFactors.Add(functor);
        }

        /// <summary>
        /// R|[lg邽߂̈oChς݂̊֐IuWFNgǉ
        /// </summary>
        /// <param name="functor">ǉ֐IuWFNg</param>
        public void AddInitializationFactor(IFunctor functor)
        {
            initializationFactors.Add(functor);
        }

        /// <summary>
        /// R|[lgˑĂR|[lg̓o^Ԃ
        /// </summary>
        /// <returns>R|[lgˑĂR|[lg̓o^</returns>
        public IComponentEntry[] GetDependencies()
        {
            ArrayList list = new ArrayList();
            foreach (object arg in injectionConstructorFunctor.GetBoundArguments())
            {
                IArgumentComponentProvider provider = arg as IArgumentComponentProvider;
                if (provider != null)
                {
                    list.Add(provider.ComponentEntry);
                }
            }
            foreach (BindFunctor functor in injectionFactors)
            {
                foreach (object arg in functor.GetBoundArguments())
                {
                    IArgumentComponentProvider provider = arg as IArgumentComponentProvider;
                    if (provider != null)
                    {
                        list.Add(provider.ComponentEntry);
                    }
                }
            }
            return (IComponentEntry[])list.ToArray(typeof(IComponentEntry));
        }

        /// <summary>
        /// R|[lg̃CX^XԂ
        /// </summary>
        /// <returns>R|[lg̃CX^X</returns>
        /// <exception cref="CyclicDependencyException">
        /// R|[lg̈ˑ֌WzĂĂAzĂR|[lg
        /// Prototype[h̏ꍇɔO
        /// </exception>
        /// <remarks>
        /// ̃\bhŐVCX^XԂ̃CX^XԂ
        /// őI܂B
        /// </remarks>
        public object GetInstance()
        {
            if (instantiating)
            {
                instantiating = false;
                throw new CyclicDependencyException();
            }

            instantiating = true;

            object instance = injectionConstructorFunctor == null ?
                Activator.CreateInstance(componentType) :
                injectionConstructorFunctor.Invoke();

            foreach (IFunctor functor in injectionFactors)
            {
                functor.Invoke(instance);
            }

            foreach (IFunctor functor in initializationFactors)
            {
                functor.Invoke(instance);
            }

            instantiating = false;

            return instance;
        }

        /// <summary>
        /// R|[lg̃CX^Xj
        /// </summary>
        /// <remarks>
        /// jR|[lgIDisposableĂ邱ƁB
        /// </remarks>
        public void Discard()
        {
            // ̎ނ̃Ggł͉Ȃ
        }
    }
}
